// Project 15 – Creating a Digital Thermometer

#include <avr/io.h>
#include <math.h>
#include <util/delay.h>

void startADC()
// Set up the ADC
{
    ADMUX |= (1 << REFS0);                 // Use AVcc pin with ADC
    ADMUX |= (1 << MUX2) | (1 << MUX0);    // use ADC5 (pin 28)
    ADCSRA |= (1 << ADPS1) | (1 << ADPS0); // prescaler for 1MHz (/8)
    ADCSRA |= (1 << ADEN);                 // enable ADC
}
 
void displayNumber(uint8_t value)
// Displays a number 0-9 on the seven-segment LED display
{
    switch(value)
    {
        case 0 : PORTB = 0b00111111; break; // 0
        case 1 : PORTB = 0b00000110; break; // 1
        case 2 : PORTB = 0b01011011; break; // 2
        case 3 : PORTB = 0b01001111; break; // 3
        case 4 : PORTB = 0b01100110; break; // 4
        case 5 : PORTB = 0b01101101; break; // 5
        case 6 : PORTB = 0b01111101; break; // 6
        case 7 : PORTB = 0b00000111; break; // 7
        case 8 : PORTB = 0b01111111; break; // 8
        case 9 : PORTB = 0b01101111; break; // 9
    }
}

int main(void)
{
    uint8_t tens=0;  // Holds tens digit for temperature
    uint8_t ones=0;  // Holds ones digit for temperature
    float temperature;
    float voltage;
    uint16_t ADCvalue;
    uint8_t finalTemp;	
 
    DDRB = 0b11111111; // Set PORTB to outputs
    DDRC = 0b00000000; // Set PORTC to inputs
    startADC();
    for(;;)
    {
        // Take reading from TMP36 via ADC
    	ADCSRA |= (1 << ADSC);         // Start ADC measurement
    	while (ADCSRA & (1 << ADSC) ); // Wait for conversion to finish 
    	_delay_ms(10);

	// Get value from ADC (which is 10-bit) register, store in ADCvalue
	ADCvalue = ADC;
	
        // convert reading to temperature value (Celsius)
        voltage = (ADCvalue * 5);
	voltage = voltage / 1024;
        temperature = ((voltage - 0.5) * 100);	

	// Display temperature on LED module
	finalTemp = (uint8_t)round(temperature);
	
	tens = finalTemp / 10;
	ones = finalTemp % 10;

	displayNumber(tens);  // Display tens digit
        _delay_ms(250);
	displayNumber(ones); // Display ones digit
        _delay_ms(250);

	// Turn off the LED display in preparation for the next reading
	PORTB = 0b00000000;
	_delay_ms(1000);
    }
    return 0;             
}
