// Project 24	 - Using two external interrupts
// PORTB blinks, INT0 rising interrupt, INT1 falling interrupt

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

ISR (INT0_vect)
{   // Code to be executed when interrupt INT0 triggered
    PORTB = 0b11111111;    
    _delay_ms(1000);
    PORTB = 0b00000000;   
    EIFR &= ~(1<<0);   // Set interrupt flag register for INT0 to zero
}

ISR (INT1_vect)
{   // Code to be executed when interrupt INT1 triggered
    PORTB = 0b11111111;    
    _delay_ms(2000);
    PORTB = 0b00000000;   
    EIFR &= ~(1<<1);   // Set interrupt flag register for INT1 to zero
}

void startInts()
{
    // initialize interrupt 0 (PD2/INT0/pin 4)
    // rising-edge (LOW to HIGH at pin 4)
    // initialize interrupt 1 (PD3/INT1/pin 5)
    // falling-edge (HIGH to LOW at pin 5)
    EICRA = 0b00001011; 

    // turn on interrupt INT0 and INT1
    EIMSK = 0b00000011; 
    // enable global interrupt enable flag for interrupts to be processed
    sei();
}

int main(void)
{
   // declare global variables
   // setup GPIO pins etc.
   DDRB = 0b11111111; // set PORTB register as outputs
   DDRD = 0b00000000; // set PORTD pins 4 and 5 as inputs
   // initialize interrupts
   startInts();

   for(;;)
   { 
     PORTB = 0b00000001;    
     _delay_ms(50);
     PORTB = 0b00000000;    
     _delay_ms(50);
   }
   return 0;              
}
