// Project 29 – Using CTC Timer for Longer Delay Repetitive Actions 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

uint16_t target = 1; // interval in seconds between running function "targetFunction"
uint16_t targetCount= 0;        // used to track number of seconds for CTC counter resets

void targetFunction()
{
    // do something once target duration has elapsed
    PORTB = 0b00000010;    
    _delay_ms(100);
    PORTB = 0b00000000; 
}

ISR(TIMER1_COMPA_vect)
{    // Code to be executed when interrupt triggered from CTC
    targetCount++;   // add one to targetCount
    if (targetCount == target)   // if required period of time has elapsed
    {
        TCNT1 = 0;          // reset TIMER1 back to zero
        targetFunction();   // do something
        targetCount = 0;	// reset targetCount back to zero
    }
}

void initCTC() 
// setup CTC interrupt and TIMER1
{
    OCR1A = 15625; // number of periods to watch for - 15625 = 1 second
    // turn on CTC mode and set CS10 and CS11 for /64 prescaler
    TCCR1B = 0b00000011; 
    TIMSK1 = 0b00000010; // turn on timer compare interrupt
    sei(); // turn on global interrupts
}

int main(void)
{
    DDRB = 0b11111111; // set PORTB register as outputs
    initCTC();         // setup overflow interrupt and TIMER1

    for(;;)            // do something (blink LED on PB0)
    {
        PORTB = 0b00000001;    
        _delay_ms(100);
        PORTB = 0b00000000;
        _delay_ms(100);
    }
    return 0; 
}