// Project 30 - Experimenting with Overflow Timers using Bitwise Operations

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

ISR(TIMER1_OVF_vect)
{
    // Code to be executed when interrupt triggered from TIMER1 overflow
    // For this example, quickly blink LED on PB1
    PORTB |= (1 << PORTB1);    // PB1 on
    _delay_ms(5);
    PORTB &= ~(1 << PORTB1);   // PB1 off
}

void initOVI() 
// setup overflow interrupt and TIMER1
{
    TCCR1B |= (1 << CS11);     //  set prescaler to /8 
    TIMSK1 |= (1 << TOIE1);    // enable TIMER1 overflow interrupts
    sei();                     // turn on global interrupts
}

int main(void)
{
    DDRB |= (1 << DDB0)|(1 << DDB1); // set PORTB pins 0 and 1 as outputs
    initOVI();  // setup overflow interrupt and TIMER1

    for(;;) // do something (blink LED on PB0)
    { 
        PORTB ^=(1 << PORTB0);    // toggle PB0
        _delay_ms(100);
    }
    return 0;             
}