// Project 48 - Using the MCP3008 Analog to Digital Converter IC

#include <avr/io.h>
#include <util/delay.h>

void writeMAX7219(char hexdata1, char hexdata2)
// sends two bytes in hexadecimal to MAX7219
{
    SPCR = 0b01010000; 		   // set up SPI bus for MAX7219	
    PORTB &= ~(1 << PORTB0);    // latch pin low
    SPDR = hexdata1;            // send value of hexdata1
    while(!(SPSR & (1<<SPIF))); // wait for SPI transmission to finish
    SPDR = hexdata2;            // send value of hexdata2
    while(!(SPSR & (1<<SPIF))); // wait for SPI transmission to finish
    PORTB |= (1 << PORTB0);     // latch pin high
}

void blankMAX7219()
// blanks all digits
{
    uint8_t i;
    for (i=1; i<9; i++)	    // blank all digits
    {
        writeMAX7219(i,15);
    }
}

void initMAX7219()
// setup MAX7219 for use
{
    PORTB |= (1 << 0); 		// latch pin high (SS)
    SPCR = 0b01010000; 		// set up SPI bus for MAX7219
    writeMAX7219(0x09,0xFF); // mode decode for digits (table 4 page 7 - 0x09 + 0xFF)
    writeMAX7219(0x0B,0x07); // set scan limit to 8 digits - 0x09 + 0xFF)
    writeMAX7219(0x0A,0x01); // set intensity to 8 - 0x0A + 0x08)
    writeMAX7219(0x0C,0x01); // mode display on (table 4 page 7 - 0x09 + 0xFF)
    blankMAX7219();
} 

void dispMAX7219(uint8_t digit, uint8_t number, uint8_t dp)
// displays "number" in location "digit" with decimal point on/off
// digit: 1~8 for location 1~8
// number: 0~15 for 0~9, - E, H, L, P, blank
// dp: 1 on, 0 off
{
    if (dp==1)      // add decimal point
    { 
        number = number + 128; 
    } 
    writeMAX7219(digit, number);
}

void numberMAX7219(uint32_t value)    
// Displays a number between 0-99999999
{
    uint8_t digits[9];
    uint8_t i = 1;
	
    for (i=1; i<9; i++)
    {
        digits[i]=15; // Sending 15 blanks the digit.
    }
    i = 1;
    while (value > 0)          // Continue until value > 0.
    {
        digits[i] = value % 10;  // Determine and store last digit. 
        value = value / 10;      // Divide value by 10.
        i++;
    }
    for (i=1; i<9; i++)
    {
        dispMAX7219(i, digits[i],0);
    }
 }


uint16_t readMCP3008()
// read channel 0 and return value
{
    uint8_t LSB;			
    uint8_t MSB;
    uint16_t ADCvalue;      //  holds data to return to main code
    SPCR = 0b01010010; 		// set up SPI bus for MCP3008 
	
    // SS on PB1 (15)
    PORTB &= ~(1 << PORTB1);    // latch pin LOW
	
    SPDR = 0b00000001;			// send start bit
    while(!(SPSR & (1<<SPIF))); // wait for SPI transmission to finish
	
    SPDR = 0b10000000; 			// select ADC0
    while(!(SPSR & (1<<SPIF))); // wait for SPI transmission to finish

    MSB = SPDR & 0b00000011;    // place top two bits of ADC value in MSB, ignore unwanted bits
	
    SPDR = 0b00000000; 			// request next 8 bits of data
    while(!(SPSR & (1<<SPIF))); // wait for SPI transmission to finish
    LSB = SPDR;					// place lower 8 bits of data in LSB
	
    PORTB |= (1 << PORTB1);     // latch pin high
	
    ADCvalue = MSB << 8 | LSB;  // construct final 10-bit ADC value
	
    return ADCvalue;
}

int main(void)
{
    uint16_t ADCoutput; 
    DDRB = 0b11111111;   // set PORTB as outputs	
    initMAX7219();
    while (1) 
    {
        ADCoutput = readMCP3008();
        ADCoutput = ADCoutput * 4.8828;  // convert ADC value to millivolts
        numberMAX7219(ADCoutput);
        _delay_ms(100);
    }
}