// Project 49 - Controlling MCP23017 outputs

#include <avr/io.h>
#include <util/delay.h>

void I2Cenable()
// enable I2C bus
{
    TWBR = 72; // 100 kHz I2C bus
    TWCR |= (1 << TWEN); // enable I2C on PORTC4 and 5
}

void I2Cwait()
// wait until I2C finishes an operation
{
    // wait until bit TWINT in TWCR is set to 1
    while (!(TWCR & (1<<TWINT))); 
}

void I2CstartWait(unsigned char address)
{
    uint8_t status;
    while (1)
    {
        // send START condition
        TWCR = (1<<TWINT) | (1<<TWSTA) | (1<<TWEN);
    
        // wait until transmission completed
        I2Cwait();
    
        // check value of TWSR, and mask out status bits
        status = TWSR & 0b11111000;
    	if ((status != 0b00001000) && (status != 0b00010000)) continue;
    
    	// send device address
    	TWDR = address;
    	TWCR = (1<<TWINT) | (1<<TWEN);
    
    	// wait until transmission completed
    	I2Cwait();
    
    	// check value of TWSR, and mask out status bits
    	status = TWSR & 0b11111000;
    	if ((status == 0b00100000 )||(status == 0b01011000)) 
    	{    	    
            // slave device is busy, so send stop condition to terminate write operation
            TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTO);
	        
            // wait until stop condition is executed and I2C bus released
            while(TWCR & (1<<TWSTO));
            continue;
        }
        break;
    }
}

void I2Cstop()
// stop I2C bus and release GPIO pins
{
// clear interrupt, enable I2C, generate stop condition
TWCR |= (1 << TWINT)|(1 << TWEN)|(1 << TWSTO);
}

void I2Cwrite(uint8_t data)
// send 'data' to I2C bus
{
    TWDR = data;
    TWCR |= (1 << TWINT)|(1 << TWEN);
    I2Cwait();
}

void initMCP23017()
// configure MCP23017 ports for all outputs
{
    I2CstartWait(0x20<<1); // 0x20 write mode
    I2Cwrite(0x00);	// IODIRA register
    I2Cwrite(0x00);	// all register A to outputs
    I2Cstop();
	
    I2CstartWait(0x20<<1); // 0x20 write mode
    I2Cwrite(0x01);	// IODIRB register
    I2Cwrite(0x00);	// all register B to outputs
    I2Cstop();
}

int main(void)
{
    uint8_t i; 

    I2Cenable();	
    initMCP23017();
	
    while (1) 
    {
        for (i = 0; i< 256; i++)
        {
            I2CstartWait(0x20<<1);
            I2Cwrite(0x12);  // control register A 0x12
            I2Cwrite(i);     // value to send 
            I2Cstop();
            I2CstartWait(0x20<<1);
            I2Cwrite(0x13);  // control register B 0x13
            I2Cwrite(i);     // value to send 
            I2Cstop();
            _delay_ms(100);			
        }
    }
}